package ru.dbotthepony.mc.otm.menu.tech

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.ForgeHooks
import net.minecraftforge.common.capabilities.ForgeCapabilities
import ru.dbotthepony.mc.otm.block.entity.RedstoneSetting
import ru.dbotthepony.mc.otm.block.entity.tech.ChemicalGeneratorBlockEntity
import ru.dbotthepony.mc.otm.core.ifPresentK
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.MatterySlot
import ru.dbotthepony.mc.otm.menu.input.EnumInputWithFeedback
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class ChemicalGeneratorMenu @JvmOverloads constructor(id: Int, inv: Inventory, tile: ChemicalGeneratorBlockEntity? = null)
	: MatteryMenu(MMenus.CHEMICAL_GENERATOR, id, inv, tile) {

	val redstone = EnumInputWithFeedback(this, RedstoneSetting::class.java)

	init {
		if (tile != null) {
			redstone.with(tile.redstoneControl::redstoneSetting)
		}
	}

	val container = tile?.container ?: SimpleContainer(3)

	val fuelSlot = object : MatterySlot(container, ChemicalGeneratorBlockEntity.SLOT_INPUT) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return ForgeHooks.getBurnTime(itemStack, null) > 0
		}
	}

	val residueSlot = object : MatterySlot(container, ChemicalGeneratorBlockEntity.SLOT_RESIDUE) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return false
		}
	}

	val batterySlot = object : MatterySlot(container, ChemicalGeneratorBlockEntity.SLOT_BATTERY) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			itemStack.getCapability(ForgeCapabilities.ENERGY).ifPresentK {
				return it.canReceive()
			}

			return false
		}
	}

	val progress = ProgressGaugeWidget(this) { 1f - tile!!.workTicks.toFloat() / tile.workTicksTotal }
	val energy = LevelGaugeWidget(this, tile?.energy)
	var burnTime by mSynchronizer.int()


	override val storageSlots = listOf(
		addSlot(fuelSlot),
		addSlot(batterySlot),
		addSlot(residueSlot),
	)

	init {
		addInventorySlots()
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		progress.updateServer()
		burnTime = (tile as ChemicalGeneratorBlockEntity).workTicks
	}
}
